import 'package:json_annotation/json_annotation.dart';
import 'package:intl/intl.dart';
part 'amount.g.dart';

NumberFormat formatter = NumberFormat("##,##,##,000");

@JsonSerializable(explicitToJson: true)
class Amount {
  final double amount;
  final String currency;

  Amount(this.amount, this.currency);
  Amount.named({this.amount, this.currency});

  Amount copyWith({double amount, String currency}) {
    return Amount.named(
        amount: amount ?? this.amount, currency: currency ?? this.currency);
  }

  operator +(Amount other) {
    assert(other.currency.toLowerCase() == currency.toLowerCase());
    return Amount.named(
        amount: (amount ?? 0) + other.amount, currency: currency);
  }

  Amount operator *(double scale) {
    return this.copyWith(amount: (amount * scale).toDouble());
  }

  Map<String, dynamic> toJson() =>
      {"currency": currency, "amount": (amount * 100).toInt()};

  factory Amount.fromJson(json) => Amount.named(
      amount: int.tryParse(json["amount"]) != null
          ? int.tryParse(json["amount"]) / 100
          : null,
      currency: json["currency"]);

  String toString() =>
      "${NumberFormat().simpleCurrencySymbol(currency)} ${formatter.format(amount)}";
}
